grammar Endpoints;
document                : generatorconfig? (namedTypeDeclaration|endpoint)* ;
generatorconfig         : '{' (configitem)? (',' configitem)* '}';
configitem              : configkey ':' configvalue ;
configkey               : IDENTIFIER ;
configvalue             : (IDENTIFIER|VALUE) ;
namedTypeDeclaration    : typeName typeDeclaration ;
typeName                : IDENTIFIER ;
typeDeclaration         : '(' typeField (',' typeField)* ')' ;
typeField               : fieldName ':' fieldType ;
fieldName               : IDENTIFIER ;
fieldType               : IDENTIFIER ;
requestBody             : REQUEST_PREFIX  (namedTypeDeclaration | typeDeclaration | IDENTIFIER) ;
responseBody            : RESPONSE_PREFIX (namedTypeDeclaration | typeDeclaration | IDENTIFIER) ;
endpoint                : path requestBody responseBody?;
path                    : (pathSegment)+ ;
pathSegment             : SLASH (IDENTIFIER|VALUE) ;


fragment DIGIT          : [0-9] ;
fragment LOWERCASE      : [a-z] ;
fragment UPPERCASE      : [A-Z] ;
fragment GENERICS       : '['|']'|'<'|'>' ;
fragment DOT            : '.' ;
fragment COMMENT_BEGIN  : '/*' ;
fragment COMMENT_END    : '*/' ;
WS                      : [ \t\n\r]+ -> skip;
COMMENT                 : COMMENT_BEGIN ~[/]* COMMENT_END -> skip;
REQUEST_PREFIX          : '<-' ;
RESPONSE_PREFIX         : '->' ;
SLASH                   : '/' ;
IDENTIFIER              : (LOWERCASE | UPPERCASE) (LOWERCASE | UPPERCASE | DIGIT | GENERICS | DOT)* ;
VALUE                   : ~[ ,{}:()/="#';*\n\r\t]+  ;
